package com.climate.data.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.climate.common.core.utils.SnowflakeIdWorker;
import com.climate.data.domain.CliTqStation;
import com.climate.data.mapper.CliTqStationMapper;
import com.climate.data.service.CliTqStationService;
import com.climate.data.utils.QueryUtils;
import com.climate.data.utils.TqConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.ListUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * @author 小p同学
 */
@Service
@Slf4j
public class CliTqStationServiceImpl implements CliTqStationService {
    @Autowired
    private CliTqStationMapper cliTqStationMapper;
    @Autowired
    private TqConfig tqConfig;

    @Override
    public int insertGeom(CliTqStation cliTqStation) {
        return cliTqStationMapper.insertGeom(cliTqStation);
    }

    @Override
    public int insertBatch(List<CliTqStation> cliTqStations) {
        return cliTqStationMapper.insertBatch(cliTqStations);
    }

    @Override
    @Transactional
    public List<CliTqStation> listSta(String adminCodes) {
        /** 2. 调用方法的参数定义，并赋值
         /** 2.1  接口ID */
        // 根据时间跟地区获取实时数据
        String interfaceId = "getSurfEleInRegionByTime";

        /** 2.2  接口参数，多个参数间无顺序 */
        HashMap<String, String> params = new HashMap<String, String>();

        /** 必选参数  资料代码 */
        params.put("dataCode", "SURF_CHN_MUL_DAY");
        /**
         * 检索时间
         * */
        /**检索时间* */
        params.put("times", "20240114000000");
        // 检索站号
        params.put("adminCodes", adminCodes);
        /**
         * 检索要素：、省名、地市名、站名、区县名、区站号(字符)、行政编码、经度、纬度
         * */
        params.put("elements", "Province,City,Station_Name,Cnty,Station_Id_C,Admin_Code_CHN");

        /**
         * 可选参数*/
        /**排序：按照站号从小到大*/

        /** 2.4 返回文件的格式 */
        String dataFormat = "json";
        /** 2.5 返回字符串 */
        StringBuffer retStr = new StringBuffer();
        List<JSONObject> objectList = new ArrayList<>();
        // 返回出去的集合
        List<CliTqStation> stationList = new ArrayList<>();
        /** 3. 调用接口 */
        try {
            //调用接口
            int rst = QueryUtils.queryClient.callAPI_to_serializedStr(tqConfig.getUsername(), tqConfig.getPassword(), interfaceId, params, dataFormat, retStr);
            //输出结果
            //正常返回
            if (rst == 0) {
                JSONObject json = JSONObject.parseObject(retStr.toString());
                System.out.println(json);
                objectList.add(json);
                /**将原数据存入，再分别存其他字段*/
                /**获取第一层键*/
                JSONArray jsonArray = json.getJSONArray("DS");
                for (Object o : jsonArray) {
                    JSONObject jsonObject = JSONObject.parseObject(o.toString());
                    CliTqStation station = new CliTqStation();
                    // 手动获取无法自动处理的
                    String province = jsonObject.getString("Province");
                    String city = jsonObject.getString("City");
                    String stationName = jsonObject.getString("Station_Name");
                    String cnty = jsonObject.getString("Cnty");
                    String stationIdC = jsonObject.getString("Station_Id_C");
                    String adminCodeChn = jsonObject.getString("Admin_Code_CHN");
                    station.setId(SnowflakeIdWorker.generateId());
                    station.setProvince(province);
                    station.setCity(city);
                    station.setCnty(cnty);
                    station.setStationId(stationIdC);
                    station.setAdminCode(adminCodeChn);
                    station.setStationName(stationName);
                    station.setType(1);
                    stationList.add(station);
                }
                //切分数据
                List<List<CliTqStation>> partition = ListUtils.partition(stationList, 200);
                System.out.println(partition);
                for (int i = 0; i < partition.size(); i++) {
                    List<CliTqStation> weaTqstaList = partition.get(i);
                    int insert = cliTqStationMapper.insertBatch(weaTqstaList);
                    if (insert > 0) {
                        System.out.println("插入成功");
                    } else {
                        System.out.println("插入失败");
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("序列化失败");
        }
        return stationList;
    }
}
